/*************************************************************************
***** Programm: BIER.EXE
***** Hauptprogramm: DRU_MEMP.C
***** Ruft aus der Hauptschleife die Unterprogramme auf,
***** Regelt die Temperatur- und Druckabfragen,
***** steuert die Relais
*****
***** Unterprogramme: TABELL2.C, KREIS.C, ABLAUF.C
*****
***** Autoren: Michael und Markus Fey
***** Erstellt am: 27.12.1996
***** Update: 29.1.1997
*************************************************************************/

#include <conio.h>
#include <dos.h>
#include <stdio.h>
#include <bios.h>
#include <time.h>
#include <math.h>

/*** Die Adresse der Druckersschnittstelle kann verschieden sein.
**** Laptop:  03BC (Daten) 03BD (Status) 03BE (Steuer)
**** Mein PC: 0378 (Daten) 079D (Status) 037A (Steuer)
***/
#define Par_Sst 0x3BC
#define lpt1_stat 0x3BD  /* Adresse des Statusregisters*/
								 /* der Parallel-Schnittstelle */
/* Hardware-Konstanten */
#define Torzeit 0x80  /* Bit 7 des Statusregisters  */
#define ImpulsDruck  0x10  /* Bit 5  "        "          */
#define ImpulsTemp  0x20  /* Bit 5  "        "          */

#define n 1

#define ESC 0x11b
#define RETURN 0x1c0d
#define TABULATOR 0xf09

#define F1 0x3b00
#define F2 0x3c00
#define F3 0x3d00
#define F4 0x3e00
#define F5 0x3f00
#define F6 0x4000
#define F7 0x4100
#define F8 0x4200
#define F9 0x4300
#define F10 0x4400

#define ALT_F1 0x6800
#define ALT_F2 0x6900
#define ALT_F3 0x6a00
#define ALT_F4 0x6b00
#define ALT_F5 0x6c00
#define ALT_F6 0x6d00
#define ALT_F7 0x6e00
#define ALT_F8 0x6f00
#define ALT_F9 0x7000
#define ALT_F10 0x7100

#define SHIFT_F1 0x5400
#define SHIFT_F2 0x5500
#define SHIFT_F3 0x5600
#define SHIFT_F4 0x5700
#define SHIFT_F5 0x5800
#define SHIFT_F6 0x5900
#define SHIFT_F7 0x6000
#define SHIFT_F8 0x6100
#define SHIFT_F9 0x6200
#define SHIFT_F10 0x6300

#define AN 1
#define AUS 0

#define HEIZUNG  16
#define WASSER 2
#define MOTOR_L 4
#define MOTOR_R 8
#define PUMPEN 1
#define SPUELEN 32
#define CHECKEN 64

#define LAENGE 111
#define SAETZE 50

extern struct {
	  char name[20+1];
	  int liter;
	  int malz;
	  int hopfen;
	  int eiw_temperatur;
	  int eiw_gesamtzeit;
	  int eiw_motorzeit;
	  int eiw_motorintervall;
	  int malt_temperatur;
	  int malt_gesamtzeit;
	  int malt_motorzeit;
	  int malt_motorintervall;
	  int zucker_temperatur;
	  int zucker_gesamtzeit;
	  int zucker_motorzeit;
	  int zucker_motorintervall;
	  int koch_zeit;
	  int koch_temperatur;
     int koch_unterbrechung;
	}
	zeile[SAETZE];


int Nummer, Zaehler = 0;
char datenbyte = 0;
char Drehrichtung = 'L';

int Unterbrechung();
int tempmessalt();
int tempmess();
int druckmess();
int erg_taste(int Taste);
void schalten(int relais, char Schalter);
int Motor( char Schalter );
void ZeigBits( char cWert );
int pflege_datei();
int hol_satz( int zaehlsatz );
void startbildschirm();
int check_it();
void ZeitSteuerung();
void Meldung ( char MeldeText[74] );
void WasserZulauf(int SollLiter, int Spuelen);
void WasserAblauf();
int AufHeizen( int SollTemp );
int TempPruefung();
void AblaufBildschirm();
void Warnton();
void Hinweiston();
int EndeFal();
int BreakTest();
int DruckFormel();

int main(int Parameter)
{

  int Taste, i, j = 4;
  double tempera;
  biosprint(1,0,0);
  if (Parameter == 1) startbildschirm();
  clrscr();
  gotoxy(5,11);
  printf("Das Programm testet die angeschlossene Braumaschine ...");
  delay(1000);
  if (check_it() == 1) return 0;
  clrscr();
  pflege_datei();
  clrscr();
  gotoxy(27,2);
  cprintf("Auswahl der Bierrezepte");
  for(i=0;j<23;i++)
		{
		if( zeile[i].name[0] != '.' )
			{
				gotoxy(5,j);
				cprintf("%d", i);
				gotoxy(10, j);
				cprintf("%s",zeile[i].name);
				gotoxy(35,j);
				cprintf("%d Liter, %d kg Malz, %d g Hopfen", zeile[i].liter,
												zeile[i].malz, zeile[i].hopfen);
				j++;
			}
		}
  Meldung("Bitte Nummer einer Biersorte auswhlen!");
  gotoxy(47,25);
  scanf( "%d", &Nummer );
  if( hol_satz(Nummer) == 999 )
	  return 999;
  ZeitSteuerung();
  outportb(Par_Sst,0);
  /* Dateien schlieen */
  return (0);
}

/********************************************************/
/* Unterprogramm zur Temperaturmessung                  */
/********************************************************/

int tempmess()
{
int ZaehlerA,ZaehlerB,i,Ausgang;
float SummeA = 0, ErgebnisA=0.00;
float SummeB = 0, ErgebnisB=0.00;
float Endergebnis = 0.00;
  for(i=0;i<1000;i++)
  {
		M0:
			asm{
				mov ZaehlerA,0
				mov ZaehlerB,0
				mov dx,lpt1_stat
			 }
		M1:
			asm{
				  in al,dx
				  test al,ImpulsTemp
				  jnz M1
				 }
		 M2:
			 asm{
				  in al,dx
				  test al,ImpulsTemp
				  jz M2
				 }
		 M3:
			 asm{
					in al,dx
					test al,ImpulsTemp
					jz M4
					inc ZaehlerA
					jmp M3
				 }
		 M4:
		 asm{
					in al,dx
					test al,ImpulsTemp
					jnz M5
					inc ZaehlerB
					jmp M4
				}
		 M5: SummeA = ErgebnisA + ZaehlerA;
			  ErgebnisA = SummeA;
			  SummeB = ErgebnisB + ZaehlerB;
			  ErgebnisB = SummeB;

	 }
/*		 gotoxy(5,20);
		 cprintf("ZaehlerImp: %.2f  ",ErgebnisA);
		 gotoxy(5,21);
		 cprintf("ZaehlerAus: %.2f  ",ErgebnisB);
		 gotoxy(5,22);
		 Endergebnis = ((ErgebnisA/ErgebnisB) * 100.00);
		 cprintf("Zahl: %.4f  ",Endergebnis);
		 gotoxy(5,23);
		 Endergebnis = ( (((ErgebnisA/ErgebnisB) * 100)-7.7)*(85.00/315.60)); */
		 Endergebnis = (pow((((ErgebnisA/ErgebnisB) * 100)-58.5),0.45)*7.5);
		 Ausgang = (Endergebnis * 10);
/*		 cprintf("Ergebnis: %.4f  ",Endergebnis);
		 gotoxy(5,24);
		 Endergebnis = pow(3,2);
		 cprintf ("Test: %.4f   ", Endergebnis);
*/
		return(Ausgang);
}


/********************************************************/
/* Unterprogramm zur Temperaturmessung                  */
/********************************************************/

int tempmessalt()
{
 int Flag;
 int Zaehler;
 asm{                        	
		mov Zaehler,0	/*   Initialisierung    */
		mov Flag,0
		mov dx,lpt1_stat
    }
 M1:
 asm{
		in  al,dx
		test al,Torzeit
		jnz M1
    }
 M2:
 asm{
		in al,dx
		test al,Torzeit
		jz M2
    }
 M3:
 asm{
		in al,dx
		test al,Torzeit
		jz M5
		test al,ImpulsTemp
		jnz M4
		mov Flag,0
		jmp M3
    }
 M4:
 asm{
		cmp Flag,1
		je M3
		inc Zaehler
		mov Flag,1
		jmp M3
    }
 M5:     /*       asm mov ax,Zaehler */
						/* Rckgabe der */
						/* Drehzahl ans Hauptprogramm */
		/* asm ret; */
	return(Zaehler);
}

/********************************************************/
/* Unterprogramm zur Druckmessung                       */
/********************************************************/

int druckmess()
{
 int Flag;
 int Zaehler;
 asm{
		mov Zaehler,0	/*   Initialisierung    */
		mov Flag,0
                mov dx,lpt1_stat
    }
 M1:
 asm{
		in  al,dx
		test al,Torzeit
		jnz M1
    }
 M2:
 asm{
		in al,dx
		test al,Torzeit
		jz M2
    }
 M3:
 asm{
		in al,dx
		test al,Torzeit
		jz M5
		test al,ImpulsDruck
		jnz M4
		mov Flag,0
		jmp M3
    }
 M4:
 asm{
		cmp Flag,1
		je M3
		inc Zaehler
		mov Flag,1
		jmp M3
    }
 M5:            asm mov ax,Zaehler /* Rckgabe der */
                     /* Drehzahl ans Hauptprogramm */
		/* asm ret; */
	return(Zaehler);
}

/***************************************************************************
**** Schaltet Relais ein und aus
***************************************************************************/

void schalten(int relais, char Schalter)
{
	if (Schalter == AUS)
		{
		if ((datenbyte & relais) == relais)
			datenbyte = datenbyte ^ relais;
		}
	else
		datenbyte = datenbyte | relais;

	outportb(Par_Sst,datenbyte);
	delay(1000);
}

/***************************************************************************
**** Prft, ob Taste gedrckt wurde
***************************************************************************/

int Unterbrechung()
{
	if( bioskey( 1 ) != 0 )
	{
		return( bioskey( 0 ));
	}
	else
		return 0;
}

/***************************************************************************
**** Prft, welche Taste gedrckt wurde
***************************************************************************/

int erg_taste(int Taste)
{
			switch(Taste)
				{
					case ESC: EndeFal(); break;
					case F1: Meldung("Unterbrechungstaste war F1"); break;
					case F2: Meldung("Unterbrechungstaste war F2"); break;
					case F3: Meldung("Unterbrechungstaste war F3"); break;
					case ALT_F1: Meldung("Unterbrechungstaste war ALT_F1"); break;
					case ALT_F2: Meldung("Unterbrechungstaste war ALT_F2"); break;
					case ALT_F3: Meldung("Unterbrechungstaste war ALT_F3"); break;
					case SHIFT_F1: Meldung("Unterbrechungstaste war SHIFT_F1"); break;
					case SHIFT_F2: Meldung("Unterbrechungstaste war SHIFT_F2"); break;
					case SHIFT_F3: Meldung("Unterbrechungstaste war SHIFT_F3"); break;
					default: Meldung("DEFAULT");break;
				}
	return 0;
}

/***************************************************************************
**** Gibt einzelne Bits der Datenleitungen auf Bildschirm aus
***************************************************************************/

void ZeigBits( char cWert )
{
	int i;
	char Ausgabe[8+1];

	Ausgabe[8] = '\0';
	for ( i=7; i>= 0; i-- )
	{
		if( cWert % 2 )
			Ausgabe[i] = '1';
		else
			Ausgabe[i] = '0';

		cWert = cWert / 2;
	}
	printf( "Bits: <%s>\n", Ausgabe );
}

/***************************************************************************
**** Prft, ob Motor bereits luft, und schaltet im Wechsel zwischen Links-
**** und Rechtslauf den Motor an oder aus. Bei bereits gesetztem Motor-Bit
**** wird das Programm abgebrochen.
***************************************************************************/

int Motor( char Schalter )
{
	int Relais;

	if( Schalter == AN )
	{
		if (( (datenbyte & MOTOR_R) == 0 ) && ( (datenbyte & MOTOR_L)  == 0 ))
		{
			if( Drehrichtung == 'L' )
			{
				Drehrichtung = 'R';
				Relais = MOTOR_R;
			}
			else
			{
				Drehrichtung = 'L';
				Relais = MOTOR_L;
			}
			schalten( Relais, AN );
		}
		else
			{
				printf( "Motor luft bereits !!! PROGRAMMABBRUCH\n" );
				return 1;
			}
	}
	else
	{
		if( Drehrichtung == 'L' )
		{
			Relais = MOTOR_L;
		}
		else
		{
			Relais = MOTOR_R;
		}
		schalten( Relais, AUS );
	}
	return 0;
}

/***************************************************************************
**** Checkt, ob die Braumaschine angeschlossen ist (ber Datenleitung 8)
**** Solange hier kein Signal gesetzt wird, schalten die Relais nicht
**** (Hardware-Sicherung)
***************************************************************************/

int check_it()
{
int datentest,Ergebnis1=0,Ergebnis2=0,Ergebnis3=0;
long i;

	outportb(Par_Sst,datenbyte);
	schalten(CHECKEN,AN);
	for (i=0;i<300000 && (Ergebnis1 + Ergebnis2 + Ergebnis3) < 3;i++)
		{
			datentest = biosprint(2,0,0);
			if(datentest & Torzeit);
			else	Ergebnis1 = 1;
			if(datentest & ImpulsDruck)
				Ergebnis2 = 1;
			if(datentest & ImpulsTemp)
				Ergebnis3 = 1;
		}
	if (i == 300000 || (Ergebnis1 + Ergebnis2 + Ergebnis3) < 3)
		{
			i=1;
			Meldung("Die Platine liefert keine Steuersignale. Bitte kontrollieren!");
			Warnton();
			delay(4000);
			i=1;
		}
	else
		i=0;
	return (i);

}

/***************************************************************************
*** Funktion, um den Programmablauf zu unterbrechen
****************************************************************************/
int EndeFal()
{
	int ende = 0, Taste;

	Meldung("Sie haben den Programmablauf unterbrochen! Weiter mit bel. Taste ...");
	Warnton();
	getch();
	Meldung("Fortsetzen (RETURN) - Nchster Brauablauf (TABULATOR) - Abbrechen (ESC)");
	for(; ende == 0;)
		{
			delay(500);
			Taste = Unterbrechung();
			switch(Taste)
				{
					case ESC: ende = 3; break;
					case TABULATOR: ende = 2; break;
					case RETURN: ende = 1; break;
				}
		}
	return (ende);
}




/***************************************************************************
****************************************************************************/
